#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Import the fake gl modules
"""
from pyglet_helper.test.fake_gl import GL_COLOR_BUFFER_BIT, GL_CULL_FACE, \
    GL_DEPTH_BUFFER_BIT, GL_DEPTH_TEST, GL_LIGHTING, GL_LIGHT0, GL_LIGHT1, \
    GL_LIGHT2, GL_LIGHT3, GL_LIGHT4, GL_LIGHT5, GL_LIGHT6,  GL_LIGHT7, \
    glClear, glClearColor, glColor3f, glEnable, GLfloat, glGenLists, \
    glLoadIdentity, glNormal3f, glVertex3f
