""" pyglet_helper.util.color defines a few standard colors
"""
from pyglet_helper.util.rgba import Rgb

BLUE = Rgb(red=0.0, green=0.0, blue=1.0)
RED = Rgb(red=1.0, green=0.0, blue=0.0)
YELLOW = Rgb(red=1.0, green=1.0, blue=0.0)
ORANGE = Rgb(red=1.0, green=0.5, blue=0.0)
BLACK = Rgb(red=0.0, green=0.0, blue=0.0)
GREEN = Rgb(red=0.0, green=1.0, blue=0.0)
CYAN = Rgb(red=0.0, green=1.0, blue=1.0)
MAGENTA = Rgb(red=1.0, green=0.0, blue=1.0)
WHITE = Rgb(red=1.0, green=1.0, blue=1.0)
PURPLE = Rgb(red=0.5, green=0.0, blue=0.5)
GRAY = Rgb(red=0.1, green=0.1, blue=0.1)
