#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Defines globally-available subpackages and symbols for the pyglet_helper
package.
"""

from __future__ import absolute_import

from . import objects
from . import common
from . import util

__version__ = "0.0.1"
__author__ = "cholloway"

__title__ = "pyglet_helper"
__description__ = "3d modelling library"
__uri__ = "http://catherineh.github.io/pyglet_helper/"


__email__ = "milankie@gmail.com"

__license__ = "AGPLv3"
__copyright__ = "Copyright (c) 2014-2016 Catherine Holloway"
